#!/usr/local/bin/perl
#
# ASMprox.pl - Reads the ASM catalog and returns objects within a specific 
# search radius (proximity). 
# Written by P. R. Blanco, CASS/UCSD.

use POSIX;  # defines acos, chr, etc.
# Command line syntax:
$syntax = "# Syntax: ASMprox RA(J2000, deg) Dec(J2000, deg) Radius(deg)\n";

# Inputs:
# RA - J2000 Right Ascension of search center, in decimal degrees
# Dec - J2000 Declination of search center, in decimal degrees
# Radius - search radius, in degrees, for the ASM catalog
# Outputs:
# If ASM sources are found within the search region, they are listed to the
# standard output. Column headings and error messages all have lines
# beginning with the "#" symbol so they can be filtered by downstream software.
# The column headings are always output even if no sources are found. 

# Constants:
# ASM Catalog file - CHANGE THE LINE BELOW to point to the location/name 
# of this file. This catalog file is a local copy of the latest version
# which can be found at:
# http://rxte.gsfc.nasa.gov/docs/xte/SOF/asmcat

$asmcatfile = "./asmcat_current";  # <-- CHANGE THIS LINE

$pi = atan2(1,1) * 4;    # pi = 3.14159...
$rad = $pi / 180;        # degrees to radians

die $syntax if $#ARGV < 2;

# See if we can open the ASM catalog
open(ASMCAT, $asmcatfile) || die join("", "# Cannot open ASM catalog ", $asmcatfile, ".\n") ;

# Check for the 3 command-line arguments and assign them to variables

$pntra = $ARGV[0]; $pntdec = $ARGV[1]; $radius = $ARGV[2];

# Do any range checking on the input variables here:

die "# RA out of range\n (0.0 to 360.0)." if ($pntra > 360.0 || $pntra < 0.0);
die "# Dec out of range (-90.0 to +90.0).\n" if ($pntdec > +90.0 || $pntdec < -90.0);
die "# Radius out of range (0.0 to 90.0).\n" if ($radius <= 0.0 || $radius > 90.0);

$nasm=0;

printf("# Sources from %s within %.2f degrees of RA=%.4f, Dec=%+.4f:\n", 
        $asmcatfile, $radius, $pntra, $pntdec);
print "# ID \tSource     \t  RA    (J2000)\t  Dec   \tDist \t+/-  \tFlag\n";
while ($line = <ASMCAT> ) {
  ($catname, $catid, $catra, $catdec, $catunc, $catext, 
   $hithres, $o_hithres, $lothres, $o_lothres, $hardt, $o_hardt, $junk,
   $o_junk, $anflag) = split(" ", $line);
   
  # If the ASM source is within range, print out some of the catalog info

   if ((abs($catdec - $pntdec) < $radius) &&  # only compute $catdist if Dec is in range
     (($catdist = &deltad($pntra, $pntdec, $catra, $catdec)) < $radius)) {
       $nasm++;
       $catunc /= 3600.0;  # position uncertainty (deg)
       $catext /= 60.0;    # source extent (deg)
       printf("%4d\t%-15.15s\t%8.4f\t%+7.4f\t%5.3f\t%5.3f\t%g\n", 
       $catid, $catname, $catra, $catdec, $catdist, $catunc, $anflag);
  }  # endif (radius in range)
} # endwhile (reading lines from ASMCAT)

# ----------------------------------------------------------------------------

sub deltad  
# Returns angular distance in degrees given RA1, DEC1, RA2, DEC2 
# in degrees.
# Requires acos function
{
   local ($ra1, $dec1, $ra2, $dec2) = @_;
#  local $pi = atan2(1,1) * 4;
#  local $rad = $pi / 180;

  local $cd = sin($rad*$dec1) * sin($rad*$dec2) + 
              cos($rad*$dec1) * cos($rad*$dec2) * cos($rad*($ra1 - $ra2));
  return(acos($cd) / $rad);
} # end of deltad

