#!/usr/bin/perl
# CGI Perl script driver for HEXTErock C program. Also calls a separate CGI
# perl script (HEXTEplot) for producing an inline (and/or Postscript) sky plot 
# of the HEXTE fields of view, and a Perl script ASMprox.pl which lists
# positions of nearby ASM catalog sources.

# P. R. Blanco, CASS/UCSD

require("cgi-lib.pl") || die "Failed to find cgi-lib.pl.\n";
$ENV{'PATH'} = "";    # more secure for CGI

# Command path to HEXTErock C executable (non-interactive option)
$hexterock_exe = "./HEXTErock/HEXTErock -batch";

# Command path to ASMprox Perl script (NOT a CGI script)
$asmprox_script = "./ASMprox.pl";

# Use the cgi-lib routine to read all form variables into the global 
# associative array %in
&ReadParse;

# Strip all whitespace from the entries
foreach $key (keys(%in)) {
   $in{$key} =~ s/[ \t]+//g;
}

# Now join them together and test for suspicious characters
# (in fact anything which isn't a floating point number or the string DEF,
# but this more general trap will do for now).
$inputs = join("", values(%in));
if ($inputs =~ /[;<>\*\|&\$!#\(\)\[\]\{\}\/:'"]/) {
  print &PrintHeader;
  die "<HTML><BODY><H1>403 Forbidden</H1>\n</BODY></HTML>\n";
} 

# print the HTML header and the start of the page
print &PrintHeader;
print <<EOD;
<HTML>
<HEAD>
<TITLE> HEXTErock output</TITLE>
</HEAD>
<BODY BGCOLOR="#FFF0F0" TEXT="#000000" LINK="#FF0000" VLINK="#A00000"
BACKGROUND="">
<H2><A HREF="http://mamacass.ucsd.edu/cgi-bin/HEXTErock.html">HEXTErock</A> output:</H2>

<PRE>
EOD

# Read in the form variables from the associative array made by ReadParse
$utdate = $in{'UTdate'};
$radeg = $in{'RA'};
$decdeg = $in{'Dec'};
$roll = $in{'roll'};
$plothw = $in{'hsize'};

# Look for erroneous input
$errmsg = ""; $err = 0;

# Check that the date provided is reasonable (post-launch). The year 2075
# should be a safe upper limit on the RXTE mission!
#
if (length($utdate) == 0 || $utdate > 20751231 || $utdate < 19951230 ) {
   $err++; $errmsg=  join("", $errmsg, "<B>UT date is missing or out of range.</B><P>\n");
}
if (length($radeg) == 0 || $radeg < 0.0 || $radeg > 360.0) {
  $err++; $errmsg = join("", $errmsg, "<B>RA is missing or out of range.</B><P>\n");
}
if (length($decdeg) == 0 || $decdeg < -180.0 || $decdeg > 180.0) {
   $err++; $errmsg = join("", $errmsg, "<B>Dec is missing or out of range.</B><P>\n");
}
if (length($roll) == 0) {
  $err++; $errmsg = join("", $errmsg, "<B>Roll angle is missing or out of range.</B><P>\n");
}
if (length($plothw) == 0 || $plothw < 0.5 || $plothw >= 90.0) {
  $err++; $errmsg = join("", $errmsg, "<B>Plot size is missing or out of range.</B><P>\n");
}

if ($err > 0) {
  print "<BODY>There was a problem with HEXTErock input values:<P>\n", $errmsg;  
  print "Please go back and try again.\n";
  die "</PRE></BODY></HTML>\n";
}

# Do a dummy run with 1.5 degree rocking. This ensures that the sun-source
# angle is OK, and allows us to capture the first part of HEXTErock's output
# (including HEXTErock's determination of the default RXTE roll angle).
# If successful, HEXTErock will be called again twice (once each for 1.5 and
# 3.0 degree rocking).

$rockdeg = "1.5";

# Form a command-line string to call the HEXTErock program with all the
# parameters
$lineparams = join(" ", $utdate, $radeg, $decdeg, $rockdeg, $roll);
$command = join(" ", $hexterock_exe, $lineparams);

# Open an input pipe to receive the program's output
## open (HEXTEROCK_PIPE, join(" ", $command, "|"));  ## deprecated method
open (HEXTEROCK_PIPE, "-|") || exec $command; # more secure for CGI

$err = 0;
while ($line = <HEXTEROCK_PIPE> ) {
  if ($line =~ /^ERROR\b/i) {                # store any error and print it
    $err++;
  }
  if ($line =~ /Roll offset=(.*) =>/i) {    # just store the roll offset
    $rolldeg = $1;
  } 
  if ($line =~ /Z axis=(.*)/i) {          # just store the pos. angle
  $posangledeg = $1;
  }
  
  # If there was an error at any stage, print everything, otherwise
  # print the UT date, Julian Date and sun-source angle only
  print $line if (($err > 0) || ($line =~ /^UT date\b/i) || ($line =~ /JD=/i)
                             || ($line =~ /sun-source angle=/i));
			     
  # Was a call to VIEWING recommended?
  $viewingon = 1 if ($line =~ /VIEWING/i);
			     
} # end (while)
close(HEXTEROCK_PIPE);

# Construct the URL for VIEWING if it was recommended by HEXTErock:

if ($viewingon == 1 ) {
   $vurl = "http://heasarc.gsfc.nasa.gov/cgi-bin/Tools/viewing/viewing.pl";   
   $vquery=sprintf("mission=RXTE&Entry=%.4f%%2C+%+.4f", $radeg, $decdeg); 
   $vurl = join("", $vurl, "?", $vquery);
   printf(qq|\n\n<LI><STRONG><A HREF="%s">Call <EM>VIEWING</EM> now with RA=%.4f Dec=%+.4f</A>\n</STRONG>|, 
   $vurl, $radeg, $decdeg);
}

die "</PRE></BODY></HTML>\n" if ($err > 0);  # die if there was an error

# Draw attention to the output roll angle if the user's input was just
# "DEF" (default)
print "<STRONG>DEFAULT</STRONG> " if ($roll =~ /DEF/); 

# Always report the actual roll offset and position angle back to the user
printf("RXTE spacecraft roll offset from sun=%+.2f degrees\n", $rolldeg);
printf("(position angle of spacecraft Z axis=%.2f degrees).\n", $posangledeg);

print "</PRE>\n";
printf '<H3>HEXTE cluster background positions:</H3>', "\n";

# Now run HEXTErock twice - once for each rocking position, and store the
# cluster A and cluster B output lines in @Alines and @Blines
#
@rockvals = ("1.5", "3.0");
for ($irock = 0; $irock < 2; $irock++) {
  # Make up a command line call to HEXTErock and run it
  $rockdeg = $rockvals[$irock];
  $lineparams = join(" ", $utdate, $radeg, $decdeg, $rockdeg, $rolldeg);
  $command = join(" ", $hexterock_exe, $lineparams);
  open (HEXTEROCK_PIPE, "-|") || exec $command; # more secure for CGI
  
  # Harvest the HEXTErock output for each cluster into Alines and Blines
  while ($line = <HEXTEROCK_PIPE>) {
    if ($line =~ /^Cluster A\b/i)  {
      $line =~ s/Cluster//;
      push(@Alines, $line);
    } elsif ($line =~ /^Cluster B\b/i) {
      $line =~ s/Cluster//;
      push (@Blines, $line);
    }  
  } # end while
  close(HEXTEROCK_PIPE);
} # end for

# Print out the HEXTErock cluster position results as a table, with red
# text for cluster A and blue for cluster B
print "<TABLE BORDER=1>\n";
for ($irock = 0; $irock < 4; $irock++) {
  print "<TR><TD>";
  print qq|<FONT COLOR=#FF0000>|, $Alines[$irock],  "</FONT></TD>";
  
  print qq|<TD><FONT COLOR=#0000FF>| ,  $Blines[$irock]; 
  print "</FONT></TD></TR> \n";
}
print "</TABLE><BORDER=2>\n";


print "<H3>HEXTE cluster fields of view:</H3>\n";

# Form the CGI query string for HEXTEplot, without the plot device

$Acode = 0; $Bcode = 0;
$query = sprintf("UTdate=%s&RA=%s&Dec=%s&roll=%s&hsize=%s&Acode=%d&Bcode=%d", 
                 $utdate, $radeg, $decdeg, $rolldeg, $plothw, $Acode, $Bcode);

# Make URLs from this query string for GIF and Postscript HTTP requests
$refgif = sprintf(qq|"http://mamacass.ucsd.edu/cgi-bin/HEXTEplot?%s&dev=%s"|,$query, "GIF");
$refcps = sprintf(qq|"http://mamacass.ucsd.edu/cgi-bin/HEXTEplot?%s&dev=%s"|,$query, "CPS");

# Include the GIF URL and provide hyperlinks to both GIF and Postscript

print <<EOP;
<TABLE BORDER=5><TR><TH>
<IMG ALT="HEXTE fields of view" SRC=$refgif WIDTH=640 HEIGHT=640>
</TH></TR></TABLE>
<P>

<STRONG>Download</STRONG>: <A HREF=$refgif>GIF</A>
<A HREF=$refcps>Postscript</A><BR>

<A HREF="./extras/HEXTErock_extra.html"><STRONG>About these sky plots</STRONG></A>,
including how to produce overlays for <A
HREF="http://skyview.gsfc.nasa.gov/"><B><EM>Skyview</EM></B></A>
output.

EOP

# ASM catalog search
$command = join(" ", $asmprox_script, $radeg, $decdeg, $plothw);
$nasm=0;
open (ASMCAT_PIPE, "-|") || exec $command; # more secure for CGI
while ($line = <ASMCAT_PIPE>) {
  if ($line !~ /#/) {
    $nasm++;
    ($catid, $catname, $catra, $catdec, $catdist, $catunc, $catflag) = split(" ", $line);
    push(@catids, $catid);
    push(@catnames, $catname);
    push(@catras, $catra);
    push(@catdecs, $catdec);
    push(@catdists, $catdist);
    push(@catuncs, $catunc);
    push(@catflags, $catflag);
  }
}

$asmcaturl = qq|"http://heasarc.gsfc.nasa.gov/docs/xte/SOF/ASM_Catalog.html"|;
$asmsrcrooturl = qq|<A HREF="http://xte.mit.edu/asmlc/srcs/|;

if ($nasm > 0) {
  if ($nasm == 1) { $s="";} else {$s = "s";}
  printf("<H3> %d <A HREF=%s>ASM catalog</A> source%s within ", $nasm, $asmcaturl, $s);
  printf("%.1f &deg; of (J2000) RA=%8.4f, Dec=%+8.4f:</H3>\n", $plothw, $radeg, $decdeg);
  
  printf("(Follow the highlighted links to view the source's 2-10 keV light curve, provided by MIT, 1 Crab = 75 count/s).<P>\n");
  
  # Print the HTML table  
  print "<TABLE BORDER=5 WIDTH=85%>";
  printf("<TR ALIGN=left><TH>ID</TH><TH>Name</TH><TH>RA</TH><TH>Dec</TH><TH>dist (&deg;)</TH><TH>+/- (&deg;)</TH></TR>\n");
  for ($i = 0; $i < $nasm; $i++) {
    if ($catflags[$i] == 1) {  # ASM flag is "on"; create a URL to MIT light curve
      $caturl = join("", $asmsrcrooturl,
                   $catnames[$i], qq|.html">|);
      printf("<TR><TD>%4d:</TD><TD>%s%-17.17s</TD><TD>%8.4f</TD><TD>%+8.4f</TD><TD>%4.2f</TD><TD>%4.2f</TD></TR>\n", 
           $catids[$i], $caturl, join("", uc($catnames[$i]), "</A>"), $catras[$i],
           $catdecs[$i], $catdists[$i], $catuncs[$i]);
     }
     else {                    # ASM flag is off; just print the source info
      printf("<TR><TD>%4d:</TD><TD>%-17.17s</TD><TD>%8.4f</TD><TD>%+8.4f</TD><TD>%4.2f</TD><TD>%4.2f</TD></TR>\n", 
           $catids[$i], uc($catnames[$i]), $catras[$i],
           $catdecs[$i], $catdists[$i], $catuncs[$i]);
     
     }
  }
  print "</TABLE><P>\n";
} else {
  print "<H3>No <A HREF=", $asmcaturl, ">ASM catalog</A> sources are in the field.</H3>\n";
}
 
# Close out the unformatted input, and add footer information to the page

print <<EOD;
<P>

<HR><A HREF="http://mamacass.ucsd.edu/hexte/index.html">
High Energy X-ray Timing Experiment</A> at UCSD<P>

</BODY>
</HTML>
EOD

exit;
