/* $Id: HEXTErock.c,v 3.13 1997/06/25 19:33:23 pblanco Exp $
 * Modification record:
 * $Log: HEXTErock.c,v $
 * Revision 3.13  1997/06/25  19:33:23  pblanco
 * Fixed bug in main() whereby radeg, decdeg where echoed before they were set.
 *
 * Revision 3.12  1997/05/27  01:30:11  pblanco
 * Updated for RXTE NRA3
 *
 * Revision 3.11  1997/05/16  21:58:08  pblanco
 * Updated messaging for RXTE Cycle 3
 *
 * Revision 3.10  1997/03/19  01:41:14  pblanco
 * changed logic in position angle calculation
 *
 * Revision 3.9  1997/03/08 02:00:58  pblanco
 * changed messaging, and added XTE-specific URL to VIEWING link
 *
 * Revision 3.8  1997/01/28  03:04:00  pblanco
 * Changed pos_att to posz_att
 *
 * Revision 3.7  1996/11/30  10:28:41  pblanco
 * new debugging information
 *
 * Revision 3.6  1996/11/20 19:31:50  pblanco
 * removed HEXTEROCK_15 and HEXTEROCK_3.0 macros
 *
 * Revision 3.5  1996/11/20 19:21:12  pblanco
 * Updated defroll_sun1 with new sun-angle limits in use by RXTE SOF
 * for some time.
 *
 * Revision 3.4  1996/10/30  02:39:19  pblanco
 * Added checks for pole pointings in posz_att.c
 *
 * Revision 3.3  1996/10/19 00:27:10  pblanco
 * new version of posz_att.c using great circle formula
 *
 * Revision 3.1  1996/10/18 02:36:57  pblanco
 * Added posz_att() and vector() functions, and associated debugging output.
 * HEXTErock now reports the position angle of RXTE's +Z axis.
 *
 * Revision 3.0  1996/09/26 20:28:13  pblanco
 * Added debugging code for quaternions
 *
 * Revision 2.25  1996/09/26  01:52:22  pblanco
 * Changed roll_attut to roll_attjd and att_utrdroll to att_jdrdroll.
 *
 * Revision 2.21  1996/09/03 20:51:43  pblanco
 * Changed default date to DEFAULT_UTDATE
 *
 * Revision 2.20  1996/07/11 21:03:05  pblanco
 * Fixed bug in att_yawzatt.c
 */ 

/* HEXTErock.c - calculates HEXTE rocking positions given a UT date,
 * J2000.0 RA and Dec pointing and an (optional) roll offset (defined
 * as the angle between the sun vector and the spacecraft X-Z plane).
 * 
 * COMPILATION/LINKING INSTRUCTIONS:
 * % cc HEXTErock.c -lm -o HEXTErock
 * 
 * Many of these routines are based on, and refer to the following book:
 * "Spacecraft Attitude Determination and Control", ed. J. R. Wertz
 * (Kluwer, 1991).
 */

/* RXTE spacecraft coordinate system, with HEXTE clusters A and B shown, along
 * with vectors specifying their +1.5-degree rocking directions (not axes). The XTE
 * look direction is along +X; this is constrained to be at least 30 degrees
 * away from the sun vector. The "roll offset", or angle between the
 * X-Z plane and the sun vector, is further constrained to a certain range
 * which at launch was +/-5 degrees, but has since been changed in orbit.
 * 
 *                      .
 *                     /|                                  X
 *                    / |                                  ^  Y
 *                   /  |                                  | /
 *                  /   |                                  |/ 
 *   _______       /   /|                           Z <----.
 *  /      /      *   / |                                 
 * /______/       |  /  |                                
 *     |      _ _ |_/___|______________                          
 *      \    / /|  B  //_(o)(o)_______/|                   
 *       \  /H/ |EXTE//0 / 1/ 2/ 3/ 4/ |    =====             
 *        \/_/  |A _//__/__/__/__/__/  |    \   /|           
 *         |/   |         P  C  A   | =======|_| |        
 *         /   /|                   | ===       \|        
 *        /   / |   |||||     ||||| | =/     ASM         
 *       *   /  |   |||||     ||||| | /                 
 *       |  /   |___________________|'
 *       | /   /|       |
 *       |/   / |       |                    XXX   XXX  TTTTTTTTT  EEEEEEEEE
 *       |   /  |    ___|____                XXX   XXX  TTTTTTTTT  EEEEEEEEE
 *       |  /   |   /   |   /                 XXX XXX      TTT     EEE
 *       | /   /   /_______/                   XXXXX       TTT     EEEEEEE
 *       |/   /                                XXXXX       TTT     EEEEEEE
 *       |   /                                XXX XXX      TTT     EEE
 *       |  /                                XXX   XXX     TTT     EEEEEEEEE
 *       | /                                 XXX   XXX     TTT     EEEEEEEEE
 *       |/
 *       '
 *				       
 * ASCII Art: Jay Sedler, RXTE Mission Operations Center.
 */

#include <stdio.h>		/* printf, sscanf */
#include <stdlib.h>		/* exit */
#include <string.h>		/* strcmp */
#include <math.h>	        /* math functions */
#include "HEXTErock.h"

/* Global variables: */
static char HEXTErockRCSversion[] = "$Revision: 3.13 $";
static char HEXTErockRCSdate[] = "$Date: 1997/06/25 19:33:23 $";
static char HEXTErockURL[] = "http://mamacass.ucsd.edu:8080/cgi-bin/HEXTErock.html";
static int HEXTErockDebugLevel = 0;
static char HEXTErockSyntax[] = 
      "HEXTErock UTDate(YYYYMMDD.dd) RA(deg) Dec(deg) RockAngle(deg) RollAngle(deg, or 'DEF')";

/* Function prototypes: not all of these functions are called by HEXTErock.c;
 * some of them are present in order to complete a library of functions for
 * other uses later, possibly by chaining them together.
 */ 
#ifdef __STDC__
ATTITUDE att_jdrdroll(double, double, double, double);/* attitude from RA, Dec and roll */
ATTITUDE att_rollxatt(double, ATTITUDE);    /* roll around X-axis */
ATTITUDE att_pitchyatt(double, ATTITUDE);   /* pitch around Y-axis */
ATTITUDE att_yawzatt(double, ATTITUDE);	    /* yaw around Z-axis */
ATTITUDE rot_axisangle(VECTOR, double);	    /* rotate attitude matrix around arbitrary inertial axis */
ATTITUDE transpose(ATTITUDE);		    /* transpose an attitude matrix (not used by hexterock.c) */
ATTITUDE att_multatt2(ATTITUDE, ATTITUDE);  /* multiply two attitude matrices */
VECTOR sc_attsky(ATTITUDE, VECTOR);	    /* Cartesian sky coords from attitude and spacecraft coords */
VECTOR sky_attsc(ATTITUDE, VECTOR);	    /* Cartesian sky coords from attitude and spacecraft coords */
VECTOR sky_rd(RADEC);			    /* Cartesian (inertial) coords from RA and Dec */
RADEC rd_sky(VECTOR);			    /* RA and Dec from cartesian (inertial) coords */
VECTOR crossp(VECTOR, VECTOR);		    /* vector cross-product */
VECTOR normvec(VECTOR);			    /* normalize a vector */
VECTOR sunpos_jd(double);		    /* Cartesian (inertial) coords of sun on a Julian Date */
VECTOR vector(double, double, double);	    /* returns a VECTOR given x, y, z */
double defroll_utsun(double, double);	    /* returns XTE default roll offset for a sun-angle and date */
double defroll_sun1(double);		    /* calculates default roll offset now used for a given sun-angle */
double getjulian(double input);		    /* convert YYYYMMDD.dd UT date to Julian */
double delta_radec2(RADEC, RADEC);	    /* angular separation of 2 RA, Dec positions */
QUATERNION q_att(ATTITUDE);		    /* quaternion from attitude matrix (not used by hexterock.c */
ATTITUDE att_q(QUATERNION);		    /* attitude matrix from quaternion (not used by hexterock.c) */
double roll_attjd(ATTITUDE, double);	    /* roll offset from attitude and Julian date */
double posz_att(ATTITUDE);                  /* position-angle of Z-axis on sky */
#else
ATTITUDE att_jdrdroll(); 
ATTITUDE att_rollxatt(); ATTITUDE att_pitchyatt(); ATTITUDE att_yawzatt(); 
ATTITUDE rot_axisangle(); ATTITUDE transpose(); ATTITUDE att_multatt2();
VECTOR sc_attsky(); VECTOR sky_attsc(); VECTOR sky_rd(); RADEC rd_sky(); 
VECTOR crossp(); VECTOR normvec(); VECTOR sunpos_jd(); VECTOR vector();
double defroll_utsun(); double defroll_sun1();
double getjulian(); double delta_radec2();
QUATERNION q_att(); ATTITUDE att_q(); double roll_attjd();
double posz_att();
#endif

/* Functions used by HEXTErock.c only (others are prototyped above): */
#ifdef __STDC__
int hexterock(double, double, double, double, double);
double getval(int, char[], int, char[], double, double, double, int);
double promptreal(char[], double, double, double);/* obtains a double value from the user */
#else
int hexterock();
double getval();
double promptreal();			    
#endif

/* MAIN PROGRAM - obtain the UT date, pointing RA and Dec, rocking (1.5 or
 * 3 degrees) and roll offset from the user; then calculate the off-axis
 * RAs and Decs.
 * REQUIREMENTS:
 * <math.h> - fabs
 * HEXTErock.h - various constants and macros
 * getval.c - check a command line value and prompt for input if necessary
 * defroll_utsun.c - calculate default XTE roll offset on a given date
 * getjulian.c - translate UT date into Julian date
 * sunpos_jd.c - solar position as a function of Julian date
 * rd_sky.c - RA and Dec from Cartesian sky vector
 * delta_radec2 - calculate angle between two RAs and Decs.
 * hexterock.c - do all the work
 */
#ifdef __STDC__
int main(int argc, char *argv[])
#else
int main(argc, argv)
int argc;		         /* number of command-line arguments */
char *argv[];		         /* string of command-line arguments */
                                 /* (and, incidentally, the ONLY explicit use 
                                  * of a POINTER in this program!!!). */
#endif
{
  double UTdate;		/* UT date in the form YYYYMMDD.dd */
  double radeg, decdeg, xrolldeg, rockdeg; /* RA, Dec, roll, and rock angle in degrees */
  double jd;
  double sunsepdeg;
  RADEC rdpoint, sunrd;
  double defrolldeg;
  int batch = 0;
  
  /* START of main program: */

  /* If the user begins the command-line arguments with "-help", print usage information, then quit. */
  if (argc > 1) {
    if ((strcmp(argv[1], "-help") == 0) || (strcmp(argv[1], "-h") == 0)) {
      printf("Syntax:\n%s\n", HEXTErockSyntax);
      return(0);
    } 
    /* See if user begins the command-line arguments with "-batch" (hidden switch) */
    if ((strcmp(argv[1], "-batch") == 0)) 
      batch = 1;
    
    /* See if the user ends the command line with "-debug" */
    if ((strcmp(argv[argc-1], "-debug") == 0))  {
      printf("(main): Debugging has been enabled on standard output.\n");
      HEXTErockDebugLevel = 1;
      argc--; 				    /* strip off this switch */
    }
  } /* endif (argc > 1) */
  
  /* parse the RCS identifiers for the date and version strings: */
  sscanf(HEXTErockRCSdate, "$Date: %s ", HEXTErockRCSdate);
  sscanf(HEXTErockRCSversion, "$Revision: %s ", HEXTErockRCSversion);
  
  printf("\n\tHEXTErock - HEXTE cluster background positions on the sky\n");
  printf("\tv%s (%s)", HEXTErockRCSversion, HEXTErockRCSdate);
  printf(" by P. R. Blanco, CASS/UCSD (pblanco@ucsd.edu)\n");
  printf("\tWWW: %s\n\n", HEXTErockURL);
  
  /* Collect input parameters either from command-line arguments in the order
   * below, or by prompting (unless in batch mode).
   */
  
  UTdate = getval(argc, argv[1 + batch], 1+batch, "UTdate (YYYYMMDD.dd)", 19951230.0, 20051231.0, (double)DEFAULT_UTDATE, batch); 
  radeg = getval(argc, argv[2 + batch], 2+batch, "RA (J2000.0, degrees)", 0.0, 360.0, 0.0, batch);
  decdeg = getval(argc, argv[3 + batch], 3+batch, "Dec (J2000.0, degrees)", -90.0, 90.0, 0.0, batch);

  jd = getjulian(UTdate);
  sunrd = rd_sky(sunpos_jd(jd));
  rdpoint.ra = radeg; rdpoint.dec = decdeg;  
  printf("UT date=%.2f: source RA=%8.4f, Dec=%+8.4f degrees (J2000)\n", UTdate, radeg, decdeg);
  printf(" =>  JD=%.2f:     sun RA=%8.4f, Dec=%+8.4f;\n", jd, sunrd.ra, sunrd.dec);
  
  /* Now calculate and print the sun angle */
  sunsepdeg = delta_radec2(rdpoint, sunrd);
  printf("\tsun-source angle=%.2f degrees.\n", sunsepdeg);

  /* Special cases causing an early end of program. */
  if (sunsepdeg <= XTESUNLIMIT) {	    /* looking too close to the sun */
    printf("ERROR: RXTE cannot observe within %.2f degrees of the sun;\n", XTESUNLIMIT);
    printf("\tthe observation cannot be made on this date.\n");
    printf("\nTo find valid RXTE observing dates, use the HEASARC's VIEWING software:\n");
    printf("\t<A HREF=\"http://heasarc.gsfc.nasa.gov/cgi-bin/Tools/viewing/viewing.pl?mission=XTE&Entry=%.4f%%2C+%+.4f\">VIEWING</A>.\n", radeg, decdeg);
    exit(0);
  }
  if ((180.0 - sunsepdeg) < XTEANTISUN) { /* looking almost anti-sun; roll offset becomes indeterminate */
    printf("ERROR: source is within %.2f degrees of the anti-sun vector on this date;\n", XTEANTISUN);
    printf("HEXTErock cannot handle this case  - change the date or contact the XTE SOF for help.\n");
    exit(0);
  }
  
  rockdeg = getval(argc, argv[4 + batch], 4+batch, "HEXTE cluster rocking angle (1.5 or 3.0 degrees only)", 1.5, 3.0, 1.5, batch);
  if ( (fabs(rockdeg - 1.5) > HEXTEROCK_TOL) && (fabs(rockdeg - 3.0) > HEXTEROCK_TOL) )
    printf("\nWARNING: 1.5 or 3.0 degrees are the only meaningful values of rocking angle!\n");

  /* Calculate the "default" roll offset from the sun whether or not it is used */
  defrolldeg = defroll_utsun(UTdate, sunsepdeg);
  if (argc > (5 + batch) 
      && ( (strcmp("DEF", argv[5 + batch]) == 0) 
      || (strcmp("def", argv[5 + batch]) == 0) ))
      xrolldeg = defrolldeg;
  else  {
    xrolldeg = getval(argc, argv[5 + batch], 5+batch, "RXTE roll offset from sun (degrees)", -(double)(XTEROLLMAX), (double)XTEROLLMAX, defrolldeg, batch);
  }
  
  printf("\n");
  hexterock(UTdate, radeg, decdeg, rockdeg, xrolldeg);
  printf("\tAll positions are accurate to within 0.1 degrees.\n");

  return(0);
}

/* defroll_utsun.c - calculates the default RXTE roll offset for an observation. */
/* REQUIREMENTS:
 * <math.h> - sin, asin
 */ 
#ifdef __STDC__
double defroll_utsun(double UTdate, double sunsepdeg)
#else
double defroll_utsun(UTdate, sunsepdeg)
double UTdate;				    /* the observation date in YYMMDD.dd */
double sunsepdeg;			    /* sun-source angle, in degrees */
#endif
{
  double defrolldeg;			    /* default roll offset */
  
  /* Select default roll offset based on the mission history to date. */
  if (UTdate < 19960321.0)  
    defrolldeg = 0.0;
  else if (UTdate < 19960322.0)
    defrolldeg = -4.5;
  else if (UTdate < 19960326.33)
    defrolldeg = 0.0;
  else if (UTdate < 19960402.5)
    defrolldeg = -4.5;
  else if (UTdate < 19960408)
    defrolldeg = 0.0;
  else if (UTdate < 19960411.77)
    defrolldeg = -8.0;
  else if (UTdate < 19960412.0)
    defrolldeg = -4.5;
  else 
    defrolldeg = defroll_sun1(sunsepdeg);
    
  return(defrolldeg);
}

/* defroll_sun1.c - calculates RXTE roll offset from sun angle. Supplied by
 * Sandy Antunes, RXTE SOF. 
 * 1996 Nov 19: Updated with new angle limits (40->41.5, 35->36.5).
 * 
 * REQUIREMENTS:
 * <math.h> - sin, asin
 * HEXTErock.h - DEG, RAD
 */ 
#ifdef __STDC__
double defroll_sun1(double dangle)
#else
double defroll_sun1(dangle)
double dangle;				    /* sun-source angle in degrees (>0) */
#endif
{
  double roll;				    /* roll offset in degrees */
  
  if (dangle > 171.0)			    /* extreme case - so use max roll */
    roll = 30.0;
  else if (dangle < 41.5)
      roll = 0.0;
  else {				    /* have to actually calculate it */
    roll = DEG * asin(sin(RAD * 8.0) / sin(RAD * dangle));
    if (roll < 0)
      roll = -roll;			    /* just to be safe */
    if (roll > (dangle - 36.5))
      roll = dangle - 36.5;
    if (roll < 0)			    /* paranoia check after the above manipulations */
      roll = 0.0;
    else if (roll > 30.0)
      roll = 30.0;
  }
  roll = 0.0 - roll;			    /* negate the result */
  return(roll);
}
    
/* getval.c - obtains a valid value from the command line, or prompts for it, 
 * unless BATCH mode is specified - in this case the value can only be obtained 
 * from the command line and if absent or invalid, the program exits.
 * 
 * REQUIREMENTS: 
 * <stdio.h> - sscanf
 * getval.c - obtains a valid value from the command line, or prompts for it, 
 * unless BATCH mode is specified - in this case the value can only be obtained 
 * from the command line and if absent or invalid, the program exits.
 */
#ifdef __STDC__
double getval(int argc, char par[], int iparm, char prompt[], double minval, double maxval,  double defaultval, int batch)
#else
double getval(argc, par, iparm, prompt, minval, maxval, defaultval, batch)
int argc;			/* number of command line arguments */
char par[];			/* command-line argument */
int iparm;			/* position on the command line */
char prompt[];			/* friendly prompt for value */
double minval, maxval, defaultval; /* max, min and default values */
int batch;			/* if true, no prompting is performed */
#endif
{
  double value;
 
  /* Different behaviours depending on whether BATCH is set (no prompting allowed) or not. */
  switch (batch) {
   case 0:			/* Interactive mode - prompt if needed */
    if (iparm >= argc)		/* value not present on command-line, so we must prompt for it */
      value = promptreal(prompt, minval, maxval, defaultval);
    else if (sscanf(par, "%lg", &value) == 0)  {   /* Try to read the command-line value */
      printf("Input value `%s' is not a number; please try again...\n", par);
      value = promptreal(prompt, minval, maxval, defaultval);
    }
    else if (value < minval || value > maxval)  {
      printf("Input value %.2f is not in the range %.2f to %.2f; please try again...\n", value, minval, maxval);
      value = promptreal(prompt, minval, maxval, defaultval);
    }
    break; 
    
   case 1:			/* BATCH mode selected - suppress all prompting */
   default:
    if (iparm >= argc) {
      printf("ERROR: %s:\n\tCommand-line value #%d is missing - exiting program.\n", prompt, iparm - batch);
      exit(0);
    } 
    else if (sscanf(par, "%lg", &value) == 0)  {   /* Read command-line value */
      printf("ERROR: %s\n\tCommand-line value of `%s' is not a number - exiting program.\n", prompt, par);
      exit(0);
    }
    else if (value < minval || value > maxval) {
      printf("ERROR: %s:\n\tCommand-line value %g is not in the range %g to %g.\nExiting program.\n", prompt, value, minval, maxval);
      exit(0);
      }
    break;
  }
  return(value);
} /* end of getval.c */

/* promptreal.c:
 * Prompts the user for a double value and returns
 * it. If the user just hits ENTER, then DEFAULTVAL is returned. If the
 * user enters the end-of-file character, execution ceases. 
 * REQUIREMENTS:
 * <stdio.h> - gets, printf, sscanf
 */
#ifdef __STDC__
double promptreal(char prompt[], double minval, double maxval, double defaultval)
#else
double promptreal(prompt, minval, maxval, defaultval)
char prompt[];			/* informative text prompt */
double minval, maxval, defaultval; /* min, max, and default values */
#endif
{
  char input[81];
  double userval;

  do {				/* execute at least once */
    printf("%s {%.2f to %.2f} [%.1f] ", prompt, minval, maxval, defaultval);
    gets(input);
    if (feof(stdin)) {		/* User hit end-of-file - exit program */
      printf("\n\tExiting program.\n");
      exit(0);
    }
    switch (input[0])  {	/* examine first non-whitespace character */
     case '\0':			/* empty string - take the default */
      return(defaultval);		    
     case '?':			/* request for help - print it out */
      printf("\tEnter a real number between %g and %g,\n\tor RETURN for the default (%.4f),\n\tor the end-of-file (EOF) character to exit.\n", minval, maxval, defaultval);
      break;
     default:			/* try interpreting all other responses as a number */
      if (sscanf(input, "%lg", &userval) == 0)  {
	printf("\tInput value `%s' is not a number, please try again...\n", input);
	break;
      }
      else if (userval > maxval || userval < minval) { /* check if entered value is in range */
        printf("\tInput value %.2f is not between %.2f and %.2f, please try again...\n", userval, minval, maxval);
	break;
      }
      else
      	return((double)userval);
    } /* end switch (*input) */

  } while (1 == 1);
} /* end of promptreal.c */

/* hexterock.c - computes HEXTE off-axis rocking positions */
/* REQUIREMENTS:
 * <stdio.h>
 * <math.h> - cos, sin, atan2, acos, floor
 * HEXTErock.h
 * int HEXTErockDebugLevel (global) - if non-zero, extra information is printed.
 */
 /* LINKING OPTIONS:
 * -lm 
 * -lsunmath (on some SunOS machines)
 */ 

#ifdef __STDC__
int hexterock(double UTdate, double radeg, double decdeg, double rockdeg, double xrolldeg)
#else
int hexterock(UTdate, radeg, decdeg, rockdeg, xrolldeg)
double UTdate;			/* UT date in the form YYYYMMDD.dd */
double radeg, decdeg;		/* J2000.0 RA and Dec in degrees */
double rockdeg;			/* off-axis angle */
double xrolldeg;		/* roll offset in degrees */
#endif
{
  ATTITUDE att;			/* spacecraft attitude matrix, as defined in Wertz eqn 12-2. */
				/* i.e. each column contains the inertial */
				/* coordinates of the spacecraft {x,y,z} axes. */
  VECTOR scaxis;		/* spacecraft axis */
  VECTOR scpoint;	        /* spacecraft pointing vector (=X-axis+boresight corrections) */
  RADEC rdpoint;		/* the corresponding RA and Dec */
  RADEC rdpointZero;		/* the on-axis RA and Dec */
  double rocksin, rockcos;	/* SINE and COSINE of rockdeg */
  double jd; 		        /* Julian Date */
  
  /* Compute the attitude matrix */
  jd = getjulian(UTdate);
  att = att_jdrdroll(jd, radeg, decdeg, xrolldeg);
  
  /* Compute and print the +Z axis position angle on the sky */
  printf("Roll offset=%+4.1f => position angle of +Z axis=%+5.1f degrees.\n\n", 
	 xrolldeg, posz_att(att));
  
  /* First reproduce the RA and Dec of the pointing (X-) axis: */
  scpoint = vector(1.0, 0.0, 0.0);
  rdpointZero = rd_sky(sky_attsc(att, scpoint));

  if (HEXTErockDebugLevel > 0) { 		    /* debugging output */
    printf("(sky_attsc/roll_attjd): Pointing (X) axis RA=%8.4f, Dec=%+8.4f, roll=%+.1f.\n", 
	   rdpointZero.ra, rdpointZero.dec, roll_attjd(att, jd));
    
    rdpoint = rd_sky(sky_attsc(att, vector(0.0, 0.0, 1.0)));
    printf("(hexterock): +Z axis (opposite ASM) RA=%8.4f, Dec=%+8.4f.\n", 
	   rdpoint.ra, rdpoint.dec);
  }
    
  /* Set up for offset rocking positions */
  rocksin = (double)sin(RAD * rockdeg); rockcos = (double)cos(RAD * rockdeg);

  /* Cluster A, "plus" position (towards -Z direction): */
  scpoint = vector(rockcos, 0.0, -rocksin);
  rdpoint = rd_sky(sky_attsc(att, scpoint));
  printf("Cluster A +%.1f degrees (-Z): RA=%8.4f, Dec=%+8.4f\n", rockdeg, rdpoint.ra, rdpoint.dec);

  /* Cluster A, "minus" position (towards +Z direction): */
  scpoint = vector(rockcos, 0.0, rocksin);
  rdpoint = rd_sky(sky_attsc(att, scpoint));
  printf("Cluster A -%.1f degrees (+Z): RA=%8.4f, Dec=%+8.4f\n", rockdeg, rdpoint.ra, rdpoint.dec);

  if (HEXTErockDebugLevel > 0) {	    /* debugging output */
    printf("(hexterock): A -%.1f computed offset angle = %.4f degrees.\n", rockdeg, delta_radec2(rdpoint, rdpointZero));
    /* Cluster A's +Z position is also a minus pitch around the Y axis. */
    scpoint = vector(1.0, 0.0, 0.0);	    /* spacecraft x-axis */
    rdpoint = rd_sky(sky_attsc(att_pitchyatt(-rockdeg, att), scpoint));
    printf("(hexterock/att_pitchyatt): A -%.1f degrees (-pitch): RA=%8.4f, Dec=%+8.4f\n\n", rockdeg, rdpoint.ra, rdpoint.dec);  
  }

  /* Cluster B, "plus" position (towards -Y direction): */
  scpoint = vector(rockcos, -rocksin, 0.0);
  rdpoint = rd_sky(sky_attsc(att, scpoint));
  printf("Cluster B +%.1f degrees (-Y): RA=%8.4f, Dec=%+8.4f\n", rockdeg, rdpoint.ra, rdpoint.dec);

  /* Cluster B, "minus" position (towards +Y direction): */
  scpoint = vector(rockcos, rocksin, 0.0);
  rdpoint = rd_sky(sky_attsc(att, scpoint));
  printf("Cluster B -%.1f degrees (+Y): RA=%8.4f, Dec=%+8.4f\n", rockdeg, rdpoint.ra, rdpoint.dec);

  if (HEXTErockDebugLevel > 0) {	    /* debugging output */
    printf("(hexterock): B -%.1f computed offset angle = %.4f degrees.\n", rockdeg, delta_radec2(rdpoint, rdpointZero));
  
    /* Cluster B's +Y position is in fact a plus yaw rotation around the Z axis. */
    scpoint = vector(1.0, 0.0, 0.0);	    /* RXTE boresight */
    rdpoint = rd_sky(sky_attsc(att_yawzatt(rockdeg, att), scpoint));
    printf("(hexterock/att_yawzatt): B -%.1f degrees (-yaw): RA=%8.4f, Dec=%+8.4f\n", rockdeg, rdpoint.ra, rdpoint.dec);  
    
    /* Or, using the more general att_multatt2(rot_axisangle()) combination 
     * (eventually, this is how one puts in the boresight correction): */
    scaxis = vector(0.0, 0.0, 1.0);	    /* spacecraft Z-axis */
    rdpoint = rd_sky(sky_attsc(att_multatt2(rot_axisangle(scaxis, rockdeg), att), scpoint));
    printf("(hexterock/rot_axisangle): B -%.2f degrees (-yaw): RA=%8.4f, Dec=%+8.4f\n", rockdeg, rdpoint.ra, rdpoint.dec);
  
    /* This also works, transforming the Z-axis into inertial coordinates 
     * and postmultiplying the attitude matrix, but you'd have to be a 
     * masochist to understand why! */
    rdpoint = rd_sky(sky_attsc(att_multatt2(att, rot_axisangle(sky_attsc(att, scaxis), -1.0*(-rockdeg))), scpoint)); 
    printf("(hexterock/att_multatt2): B -%.2f degrees (-yaw): RA=%8.4f, Dec=%+8.4f\n", rockdeg, rdpoint.ra, rdpoint.dec); 
  }  

  if (HEXTErockDebugLevel > 0)		    /* debugging output */
  /* Transform the attitude matrix to a quaternion, and back again. */  
    att = att_q(q_att(att));
  
  return(0);
 } /* end of hexterock.c */

/* att_jdrdroll.c - calculates attitude matrix given RA, Dec, date and roll offset */
#ifdef __STDC__
ATTITUDE att_jdrdroll(double jd, double radeg, double decdeg, double xrolldeg)
#else
ATTITUDE att_jdrdroll(jd, radeg, decdeg, xrolldeg)
double jd;			/* Julian Date */
double radeg, decdeg;		/* J2000.0 RA and Dec in decimal degrees */
double xrolldeg;		/* x-axis roll offset in degrees */
#endif
{
  VECTOR sunvec, xvec, yvec, zvec;
  RADEC sunrd, rdpoint;
  ATTITUDE att;
  double sunsepdeg;		/* angle between source and sun (degrees) */
  unsigned int j;
  
  sunvec = normvec(sunpos_jd(jd));
  sunrd = rd_sky(sunvec);
  
  if (HEXTErockDebugLevel > 0)  	    /* debugging output */
    printf("(att_jdrdroll): Sun vector is: x=%g y=%g z=%g.\n", sunvec.xyz[0], sunvec.xyz[1], sunvec.xyz[2]); 
  
  rdpoint.ra = radeg; rdpoint.dec = decdeg;

  /* Calculate angular distance of sun from target */
  sunsepdeg = delta_radec2(rdpoint, sunrd);

  /* Ignoring the general case of instrument boresight offsets, point the 
   * spacecraft X-axis at the source RA and Dec. */
  xvec = normvec(sky_rd(rdpoint));  

  /* For zero roll offset, the X-Z plane is aligned towards the sun. 
   * and the y-axis in this case can be found by taking the cross
   * product of the sun vector with the X-axis. */
  yvec = normvec(crossp(sunvec, xvec));
  zvec = normvec(crossp(xvec, yvec));

  /* Fill the attitude matrix columns with the spacecraft axes (Wertz eqn 12.2). */
  for (j = 0; j < 3; j++) {
    att.matrix[0][j] = xvec.xyz[j];
    att.matrix[1][j] = yvec.xyz[j];
    att.matrix[2][j] = zvec.xyz[j];
  }  
  /* Roll around the X-axis, if requested */
  /* if (xrolldeg != 0.0) */
    att = att_rollxatt(xrolldeg, att);
    
  if (HEXTErockDebugLevel > 0)  {	    /* debugging output */
    printf("(att_jdrdroll): XTE attitude matrix for this observation:\n");
    for (j = 0; j < 3; j++) 
      printf("(att_jdrdroll): %10.5f %10.5f %10.5f\n", 
	     att.matrix[j][0], att.matrix[j][1], att.matrix[j][2]);
    
    /* calculate the angle between the X-Z plane normal (=-Y) and the sunvector */
    yvec = vector(0.0, -1.0, 0.0);
    printf ( "(att_jdrdroll): X-Z plane normal is at %.2f deg from sun.\n", 
	    delta_radec2(rd_sky(sky_attsc(att, yvec)), rd_sky(sunvec)) );
  }
  
  return(att);
} /* end of att_jdrdroll.c */

/* att_rollxatt.c - rolls an attitude matrix around the X-axis. */
#ifdef __STDC__
ATTITUDE att_rollxatt(double xrolldeg, ATTITUDE att)
#else 
ATTITUDE att_rollxatt(xrolldeg, att)
double xrolldeg;		/* roll offset in degrees */
ATTITUDE att;			/* input attitude matrix */
#endif
{
  ATTITUDE result;
  unsigned int j;
  double c, s;			/* cosine and sine of roll offset */
  
  /* Transform the new Y and Z axes relative to the old.  
   * Looking along the x-axis (to the source) we see:
   * 
   *         Y
   *         |
   *         .---> Z
   *
   * ...where the X-Z plane includes the solar vector. Roll away from this
   * plane in an clockwise direction, such that if <y(3)> and <z(3)>
   * are the spacecraft axes vectors in inertial coordinates, the
   * "new" post-roll axes will be given by:
   * <y'(3)> = <y(3)> cos(roll) + <z(3)> sin(roll)
   * <z'(3)> =-<y(3)> sin(roll) + <z(3)> cos(roll)
   * ... and of course <x'(3)> = <x(3)> stays the same.
   * For purists the equivalent rotation matrix is:
   *           {1, 0, 0}
   * R(i,j) =  {0, c, s}
   *           {0,-s, c}
   * ...where c = cos(roll) and s = sin(roll)
   */
  c = cos(RAD * xrolldeg); s = sin(RAD * xrolldeg);
  for (j = 0; j < 3; j++)  {
    result.matrix[0][j] = att.matrix[0][j]; /* X-axis unchanged in space */
    result.matrix[1][j] = att.matrix[1][j] * c + att.matrix[2][j] * s; /* i.e. <y'(3)> = <y(3)> cos(roll) - <z(3)> sin(roll) */
    result.matrix[2][j] = -att.matrix[1][j] * s + att.matrix[2][j] * c;	/* i.e. <z'(3)> = <z(3)> cos(roll) + <y(3)> sin(roll) */
  }
  return(result);
} /* end of att_rollxatt.c */

/* att_pitchyatt.c - pitches an attitude matrix around the Y-axis. */
#ifdef __STDC__
ATTITUDE att_pitchyatt(double ypitchdeg, ATTITUDE att)
#else
ATTITUDE att_pitchyatt(ypitchdeg, att)
double ypitchdeg;		/* pitch offset in degrees */
ATTITUDE att;			/* input attitude matrix */
#endif
{
  ATTITUDE result;
  unsigned int j;
  double c, s;			/* cosine and sine of pitch angle */

  /* Transform the new Y and Z axes relative to the old.  
   * Looking along the y-axis we see:
   * 
   *         X
   *         |
   *    Z <--.
   *
   * The +pitch maneuver (performed by XTE's solar panels, for instance)
   * is in a clockwise direction, such that if <x(3)> and <z(3)> 
   * are the spacecraft axes vectors in inertial coordinates, the
   * "new" post-pitch axes will be given by:
   * <x'(3)> =  <x(3)> cos(pitch) - <z(3)> sin(pitch)
   * <z'(3)> =  <x(3)> sin(pitch) + <z(3)> cos(pitch)
   * ... and of course <y'(3)> = <y(3)> stays the same.
   * For purists the equivalent rotation matrix is:
   *           {c, 0,-s}
   * R(i,j) =  {0, 1, 0}
   *           {s, 0, c}
   * ...where c = cos(pitch) and s = sin(pitch)'
   */    
  c = cos(RAD * ypitchdeg); s = sin(RAD * ypitchdeg);
  for (j = 0; j < 3; j++)  {
    result.matrix[0][j] = att.matrix[0][j] * c - att.matrix[2][j] * s; /* i.e. <x'(3)> = <x(3)> cos(pitch) + <z(3)> sin(pitch) */
    result.matrix[1][j] = att.matrix[1][j]; /* Y-axis unchanged in space */
    result.matrix[2][j] = att.matrix[1][j] * s + att.matrix[2][j] * c; /* i.e. <z'(3)> = <z(3)> cos(pitch) - <x(3)> sin(pitch) */
  }
  return(result);
} /* end of att_pitchyatt.c */

/* att_yawzatt.c - yaws an attitude matrix around the spaecraft Z-axis. */
#ifdef __STDC__
ATTITUDE att_yawzatt(double zyawdeg, ATTITUDE att)
#else
ATTITUDE att_yawzatt(zyawdeg, att)
double zyawdeg;			/* yaw offset in degrees */
ATTITUDE att;			/* input attitude matrix */
#endif
{
  ATTITUDE result;
  unsigned int j;
  double c, s;			/* cosine and sine of yaw angle */
  
  /* Transform the new Y and Z axes relative to the old.  
   * Looking along the Z-axis we see:
   * 
   *         X
   *         |
   *         .--> Y
   *
   * The +yaw maneuver (performed by XTE's solar panels, for instance)
   * is in the clockwise direction, such that if <x(3)> and <y(3)> 
   * are the spacecraft axes vectors in inertial coordinates, the
   * "new" post-yaw axes will be given by:
   * <x'(3)> =   <x(3)> cos(yaw) + <y(3)> sin(yaw)
   * <y'(3)> =  -<x(3)> sin(yaw) + <y(3)> cos(yaw)
   * ... and of course <z'(3)> = <z(3)> stays the same.
   * For purists the equivalent rotation matrix is:
   *           {c,-s, 0}
   * R(i,j) =  {s, c, 0}
   *           {0, 0, 1}
   * ...where c = cos(yaw) and s = sin(yaw)
   */    
  c = cos(RAD * zyawdeg); s = sin(RAD * zyawdeg);
  for (j = 0; j < 3; j++)  {
    result.matrix[0][j] = att.matrix[0][j] * c + att.matrix[1][j] * s;   /* i.e. <x'(3)> = <x(3)> cos(yaw) - <y(3)> sin(yaw) */
    result.matrix[2][j] = -att.matrix[0][j] * s + att.matrix[1][j] * c;   /* i.e. <y'(3)> = <x(3)> sin(yaw) + <y(3)> cos(yaw) */
    result.matrix[2][j] = att.matrix[2][j]; /* Z-axis unchanged in space */
  }
  return(result);
} /* end of att_yawzatt.c */

/* rot_axisangle.c - returns a rotation matrix about a vector VEC 
 * by angle PHIDEG degrees. See Wertz eqn 12-7a for details.
 * If this rotation matrix R doesn't produce the expected results, 
 * try the transformed matrix Att # R # transpose(Att) where "#"
 * represents the matrix multiplication performed by att_multatt2.c
 */
#ifdef __STDC__
ATTITUDE rot_axisangle(VECTOR evec, double phideg)
#else
ATTITUDE rot_axisangle(evec, phideg)
VECTOR evec;			/* vector defining rotation axis */
double phideg;			/* Euler rotation angle in degrees */
#endif
/* REQUIREMENTS:
 * <math.h> - sin, cos
 * HEXTErock.h - RAD
 */
{ 
  ATTITUDE rotation;		/* structure to contain the matrix */
  double c, s;			/* cos(phi) and sin(phi) respectively */
  double e1, e2, e3;		/* rotation axis components */

  /* Normalize the vector axis */
  evec = normvec(evec);
  e1 = evec.xyz[0]; e2 = evec.xyz[1]; e3 = evec.xyz[2];
  c = cos(RAD * phideg); s = sin(RAD * phideg);

  /* first row */
  rotation.matrix[0][0] = c + e1*e1 * (1.0 - c);
  rotation.matrix[0][1] = e1*e2 * (1.0 - c) + e3*s;
  rotation.matrix[0][2] = e1*e3 * (1.0 - c) - e2*s;
  
  /* second row */
  rotation.matrix[1][0] = e1*e2 * (1.0 - c) - e3*s;
  rotation.matrix[1][1] = c + e2*e2 * (1.0 - c);
  rotation.matrix[1][2] = e2*e3 * (1.0 - c) + e1*s;
  
  /* third row */
  rotation.matrix[2][0] = e1*e3 * (1.0 - c) + e2*s;
  rotation.matrix[2][1] = e2*e3 * (1.0 - c) - e1*s;
  rotation.matrix[2][2] = c + e2*e2 * (1.0 - c);
  
return(rotation);
} /* end of rot_axisangle.c */

/* transpose.c - forms the transpose of an attitude matrix and returns the result */
#ifdef __STDC__
ATTITUDE transpose(ATTITUDE att)
#else
ATTITUDE transpose(att)
ATTITUDE att;
#endif
{
  unsigned int i, j;
  ATTITUDE result;
  for (i = 0; i < 3; i++)
    for (j = 0; j < 3; j++) 
      result.matrix[i][j] = att.matrix[j][i];

  return(result);
} /* end of transpose.c */

/* att_multatt2.c - multiplys 2 attitude matrices and returns the result. */
#ifdef __STDC__
ATTITUDE att_multatt2(ATTITUDE amat1, ATTITUDE amat2)
#else
ATTITUDE att_multatt2(amat1, amat2)
ATTITUDE amat1, amat2;		/* input matrices to be multiplied, in order */
#endif
{
  unsigned int i, j, k;
  ATTITUDE result;		/* contains calculated result matrix */
  
  /* Perform the matrix multiplication. This is equivalent to a rotation 
   * of AMAT1 by a set of direction cosines AMAT2 (see Wertz p. 431). */
  for (i = 0; i < 3; i++)
    for (j = 0; j < 3; j++)	/* sum along row i of AMAT1 times column j of AMAT2 */
      for (k = 0, result.matrix[i][j] = 0.0; k < 3; k++)
        result.matrix[i][j] += amat1.matrix[i][k] * amat2.matrix[k][j];      
  
  return(result);
} /* end of att_multatt2.c */

/* sky_attsc.c - transforms a spacecraft vector to an inertial one. */  
#ifdef __STDC__
VECTOR sky_attsc(ATTITUDE att, VECTOR scvec)
#else
VECTOR sky_attsc(att, scvec)
ATTITUDE att;			/* spacecraft attitude matrix (Wertz eqn. 12-2) */
VECTOR scvec;			/* spacecraft coordinates to be transformed */
#endif
{    
  unsigned int i, j;
  VECTOR result;
  
  /* perform a matrix transformation */
  for (i = 0; i < 3; i++)
    for (j = 0, result.xyz[i] = 0.0; j < 3; j++)
      result.xyz[i] += att.matrix[j][i] * scvec.xyz[j];	/* use transpose attitude */
  
  return(result);
} /* end of sky_attsc.c */

/* sc_attsky.c - transforms a sky (inertial) vector to spacecraft coordinates. */  
#ifdef __STDC__
VECTOR sc_attsky(ATTITUDE att, VECTOR skyvec)
#else
VECTOR sc_attsky(att, skyvec)
ATTITUDE att;			/* spacecraft attitude matrix (Wertz eqn. 12-2) */
VECTOR skyvec;			/* sky coordinates to be transformed */
#endif
{    
  unsigned int i, j;
  VECTOR result;
  
  /* Perform a matrix transformation: the result follows from the 
   * definition of attitude matrix in Wertz.
   */ 
  for (i = 0; i < 3; i++)
    for (j = 0, result.xyz[i] = 0.0; j < 3; j++)
      result.xyz[i] += att.matrix[i][j] * skyvec.xyz[j];  
  
  return(result);
} /* end of sky_attsc.c */
    
/* sky_rd.c - calculates Cartesian (inertial) coordinates from RA and Dec. */ 
#ifdef __STDC__
VECTOR sky_rd(RADEC coords)
#else
VECTOR sky_rd(coords)
RADEC coords;			/* Input RA and Dec in decimal degrees */
#endif
{    
  VECTOR result;		/* contains calculated inertial coords */
  double cra, sra;
  double cdec, sdec;

  /* Convention is that the 3-axis points to the pole at Dec=90, and that
   * the Y-axis points to the equinox at RA=0. */
  cra = cos (RAD* coords.ra); sra = sin(RAD * coords.ra);
  cdec = cos (RAD* coords.dec); sdec = sin(RAD * coords.dec);  
  result = vector(cdec * cra, cdec * sra, sdec);
  return(result);
} /* end of sky_rd.c */

/* rd_sky.c - calculates RA and Dec from Cartesian (inertial) coordinates
 * and returns them in an RADEC struct.
 */ 
#ifdef __STDC__
RADEC rd_sky(VECTOR skyvec)
#else
RADEC rd_sky(skyvec)
VECTOR skyvec;			/* input Cartesian inertial vector to be converted */
#endif
{    
  double decdeg, radeg;		/* calculated RA and Dec */
  RADEC result;			/* vector to contain them for output */
  
  /* Convention is that the 3-axis points to the pole at Dec=90, and that
   * the 2-axis points to the equinox at RA=0. */

  decdeg = 90.0 - DEG * acos(skyvec.xyz[2]);
  
  if (skyvec.xyz[0] == 0.0 && skyvec.xyz[1] == 0.0)
    {
      printf("WARNING: Pointing at the celestial pole: setting RA = 0.0 degrees\n");
      radeg = 0.0;
    }
  else {
    radeg = DEG * atan2(skyvec.xyz[1], skyvec.xyz[0]);
    if (radeg < 0.0)
      radeg += 360.0;
  }
  result.ra = radeg;
  result.dec = decdeg;
  return(result);
} /* end of rd_sky.c */

/* crossp.c - makes a 3-vector cross product and returns the result */
#ifdef __STDC__
VECTOR crossp(VECTOR vec1, VECTOR vec2)
#else
VECTOR crossp(vec1, vec2)
VECTOR vec1, vec2;		/* input vectors to be multiplied */
#endif
{
  VECTOR result;		/* holds output result; */

  /* form the cross-product */
  result.xyz[0] = vec1.xyz[1]*vec2.xyz[2] - vec1.xyz[2]*vec2.xyz[1];
  result.xyz[1] = vec1.xyz[2]*vec2.xyz[0] - vec1.xyz[0]*vec2.xyz[2];
  result.xyz[2] = vec1.xyz[0]*vec2.xyz[1] - vec1.xyz[1]*vec2.xyz[0];
  return(result);
} /* end of crossp.c */

/* normvec.c - normalizes a vector to unit magnitude. */
#ifdef __STDC__
VECTOR normvec(VECTOR vec)
#else
VECTOR normvec(vec)
VECTOR vec;			/* input vector to be normalized */
#endif
{
  double tot;
  unsigned int i;
  
  for (i = 0, tot = 0.0; i < 3; i ++)
    tot += (double)(vec.xyz[i]*vec.xyz[i]);
  tot = sqrt(tot);
  
  if (tot <= 0.0)
    tot = 1.0;
    
  for (i = 0; i < 3; i++)  {
    vec.xyz[i] /= tot;
  }  
  return(vec);

} /* end of normvec.c */
   
/* sunpos_jd.c - caclulates the J2000.0 inertial coords of the sun given a
 * Julian date. This code was translated by Tom Gasaway from the formulae
 * in:
 * Meeus, Jean: Astronomical formulae for calculators (Willmann-Bell, 1988).
 * Further modified by Philip Blanco, CASS/UCSD.
 * REQUIREMENTS:
 * <math.h> - sin, cos, tan
 * HEXTErock.h - VECTOR definition
 * HEXTErock.c - vector() function
 */ 
#ifdef __STDC__
VECTOR sunpos_jd(double jd)
#else
VECTOR sunpos_jd(jd)
double jd;			/* input Julian date */
#endif

#define DEG_TO_RAD 3.14159265358979323/180.0
#define RAD_TO_DEG 180.0/3.14159265358979323
#define SUN_EPOCH 2556.50
#define SUN_EPOCH_BIAS 0.0
#define SMSE0 23.440
#define SMSE1 0.0000004
#define SMMA1 357.528
#define SMMA2 0.9856003
#define SMML1 280.466
#define SMML2 0.9856474
#define SMEL1 1.915
#define SMEL2 0.020
#define SMGPL 1.3969713
#define SMR1 1.00014
#define SMR2 0.01671
#define SMR3 0.00014
#define RS0 1.4959787066E+8
#define JAN95 1827.5
#define JDCONVERSION 2451545.0

{
  /* Local variables: */
  double slong, SMTime, sMeanSolarLong;
  double CorrectionEM, vEM, sInc;
  double rInc, rlong, rvEM;
  VECTOR SolarInertialPos;		    /* vector to contain result */
  
  SMTime = jd - JDCONVERSION;

  sInc = SMSE0 - SMSE1*SMTime;

  vEM = SMMA1 + SMMA2*SMTime;
/* change the mean anomoly vEM into radians (so it will work in 
 * the trig functions) 
 */ 
  rvEM = DEG_TO_RAD * vEM;
  sMeanSolarLong = SMML1 + SMML2*SMTime;
  CorrectionEM = SMEL1*sin(rvEM) + SMEL2*sin(2.0*rvEM);
  slong = sMeanSolarLong + CorrectionEM;

  /* tiny correction term, for conversion to the j2000 coordinate system */
  slong = slong - (SMGPL*SMTime/36525.0);
  
/* make longitude an angle between 0 and 360 degrees */
  for (; slong > 360.0; ) 
    slong -= 360.0;
  for (; slong < 0.0; )
    slong += 360.0;
  
/* convert the longitude and inclination to radians */
  rlong = DEG_TO_RAD * slong;
  rInc = DEG_TO_RAD * sInc;

/* calculate the position of the sun as a unit vector */
  SolarInertialPos = vector(cos(rlong), sin(rlong) * cos(rInc), sin(rlong) * sin(rInc));
  
  return(SolarInertialPos);
} /* end of sunpos_jd.c */

/* vector.c - returns a VECTOR structure given the x, y, and z components */
#ifdef __STDC__
VECTOR vector(double x, double y, double z)
#else
VECTOR vector(x, y, z)
double x, y, z;			    /* vector's {x,y,z} components */
#endif   
{
  VECTOR result;
  
  result.xyz[0] = x;
  result.xyz[1] = y;
  result.xyz[2] = z;
  
  return(result);
} /* end of vector.c */

/*
 * getjulian.c
 * Input date in format YYMMDD.dd where YYYY is the year, MM the month,
 * DDdd is the day of the month with dd representing the fractional day.
 * Return output is Julian Day.
 * REQUIREMENTS:
 * <math.h> - floor, fmod
 */
#ifdef __STDC__
double getjulian(double utdate )
#else
double getjulian(utdate)
double utdate;			/* input UT date in the form YYYYMMDD.dd */
#endif
{
  double temp;
  double YYYY, MM, DD, y, m, A=0.0, B=0.0, JD;
  
  YYYY = floor(utdate / 10000.0);
  
  MM = fmod(floor(utdate/100.0), 100.0);   
  DD =  fmod(utdate, 100.0);
 
  if (MM > 2 ) {
    y = YYYY;
    m = MM;
  } else {
    y = YYYY - 1.0;
    m = MM + 12.0;
  }

  if (utdate > 1582.101500 ) {		    /* it had better be! */
    A = floor(y / 100.0 );
    B = 2 - A + floor(A / 4.0 );
  }

  if (y < 0.0 ) 
    temp = floor(y * 365.25 - 0.75 );
  else	
    temp = floor(y * 365.25 );

  JD = temp + floor((m + 1.0) * 30.6001 ) + DD + 1720994.5;
  
  if (utdate > 1582.101500 )
    JD = JD + B;

  return(JD);
} /* end of getjulian.c */

/* delta_radec2.c - returns the angular separation between 2 RA and Dec coordinates,
 * all in degrees.
 * REQUIREMENTS:
 * <math.h> - sin, cos, acos
 * HEXTErock.h - RADEC structure definition, RAD, DEG
 */ 
#ifdef __STDC__
double delta_radec2(RADEC rd1, RADEC rd2)
#else
double delta_radec2(rd1, rd2)
RADEC rd1, rd2;			/* RA, Dec of the two positions */
#endif
{
  double cosd;
  
  cosd = sin(RAD*rd1.dec) * sin(RAD*rd2.dec) 
       + cos(RAD*rd1.dec) * cos(RAD*rd2.dec) * cos(RAD*(rd1.ra-rd2.ra));
  
  return(DEG * acos(cosd));
} /* end of delta_radec2.c */

/* q_att.c - returns a quaternion calculated from an attitude matrix.
 * REQUIREMENTS:
 * HEXTErock.h - QUATERNION, ATTITUDE
 * int HEXTErockDebugLevel - prints out the quaternion when > 0
 */
#ifdef __STDC__
QUATERNION q_att(ATTITUDE att)
#else
QUATERNION q_att(att)
ATTITUDE att;
#endif
{
  QUATERNION result;
  
  result.q[3] = 0.5 * sqrt(1+ att.matrix[0][0] + att.matrix[1][1] + att.matrix[2][2]);
  
  result.q[0] = 0.25 / result.q[3]  * (att.matrix[1][2] - att.matrix[2][1]);
  result.q[1] = 0.25 / result.q[3]  * (att.matrix[2][0] - att.matrix[0][2]);
  result.q[2] = 0.25 / result.q[3]  * (att.matrix[0][1] - att.matrix[1][0]);
  
  if (HEXTErockDebugLevel > 0) 		    /* debugging output */
    printf("(q_att): Quaternion = %10.5f %10.5f %10.5f %10.5f\n", 
	   result.q[0], result.q[1], result.q[2], result.q[3]);
  
  return(result);
} /* end of q_att.c */

/* att_q.c - returns an attitude matrix calculated from an input quaternion. 
 * REQUIREMENTS:
 * HEXTErock.h - ATTITUDE, QUATERNION
 * int HEXTErockDebugLevel - prints out result if > 0
 */ 
#ifdef __STDC__
ATTITUDE att_q(QUATERNION quat)
#else
ATTITUDE att_q(quat)
QUATERNION quat;
#endif
{
  ATTITUDE result;
  double q1, q2, q3, q4;
  int i;
  
  q1 = quat.q[0]; q2 = quat.q[1]; q3 = quat.q[2]; q4 = quat.q[3];
  
  /* First row, i.e. inertial coords of spacecraft X-axis: */
  result.matrix[0][0] = q1*q1-q2*q2-q3*q3+q4*q4;
  result.matrix[0][1] = 2.0 * (q1*q2 + q3*q4);
  result.matrix[0][2] = 2.0 * (q1*q3 - q2*q4);
  
  /* 2nd row, i.e. inertial coords of spacecraft Y axis: */
  result.matrix[1][0] = 2.0 * (q1*q2 - q3*q4);
  result.matrix[1][1] = -q1*q1+q2*q2-q3*q3+q4*q4;
  result.matrix[1][2] = 2.0 * (q2*q3 + q1*q4);
  
  /* 3rd row, i.e. inertial coords of spacecraft Z axis: */
  result.matrix[2][0] = 2.0 * (q1*q3 + q2*q4);
  result.matrix[2][1] = 2.0 * (q2*q3 - q1*q4);
  result.matrix[2][2] = -q1*q1-q2*q2 + q3*q3 + q4*q4;
  
  if (HEXTErockDebugLevel > 0)  {	    /* debugging output */
    printf("(att_q): Attitude matrix is now:\n");
    for (i = 0; i < 3; i++) 
      printf("(att_q): %10.5f %10.5f %10.5f\n", 
	     result.matrix[i][0], result.matrix[i][1], result.matrix[i][2]);
  }

  return(result);
} /* end of att_q.c */

/* roll_attjd.c 
 * Computes XTE spacecraft roll offset from sun for a given attitude matrix
 * and Julian Day.
 * 
 * REQUIREMENTS:
 * HEXTErock.h - ATTITUDE, VECTOR, DEG
 * <math.h> - acos
 * crossp - vector cross product
 * normvec - normalize a VECTOR
 * sc_attsky - Spacecraft coordinates given attitude matrix and sky coords
 * sunpos_jd - solar position as a function of Julian Day
 */ 
#ifdef __STDC__
double roll_attjd(ATTITUDE att, double jd)
#else
double roll_attjd(att, jd)
ATTITUDE att;				    /* XTE spacecraft attitude matrix */
double jd;				    /* Julian Day */
#endif   
{
  VECTOR scsun, scyprime;		    /* spacecraft vectors */
  double rolloffdeg; 			    /* the result */
  
  /* Transform the sun vector into spacecraft coordinates */
  scsun = sc_attsky(att, normvec(sunpos_jd(jd)));
    
  /* Form the cross-product of sun and x-axis: the result lies in
   * the spacecraft Y-Z plane. 
   */ 
  scyprime = normvec(crossp(scsun, vector(1.0, 0.0, 0.0)));
     
  /* Find the angle between yprime and the spacecraft y-axis by taking
   * the arcsin of yprime's Z-coordinate, respecting sign conventions;
   * this appears to produce consistent results  for [-90, 90] degrees.
   */
  rolloffdeg = -1.0 * DEG * asin(scyprime.xyz[2]);

  return(rolloffdeg);
} /* end of roll_attjd.c */

/* posz_att.c - calculates the position angle of +Z axis relative to North.
 * Unlike roll_attjd.c this gives an absolute angle on the sky. 
 * A better algorithm may exist. This version simply calculates
 * the "great circle initial heading" from the spacecraft pointing axis to
 * a small angular displacement in +Z (which happens to be 1.5 degrees).
 * REQUIREMENTS:
 * <math.h> - trig functions
 * HEXTErock.h - RAD, DEG macros and RADEC, VECTOR, ATTITUDE structures.
 * HEXTErock.c - rd_sky(), sky_attsc(), vector() functions
 *             - HEXTErockDebugLevel global variable.
 */ 
#ifdef __STDC__
double posz_att(ATTITUDE att)
#else
double posz_att(att)
ATTITUDE att;
#endif
{
  double lo0, la0, lo1, la1;
  RADEC rdpoint0, rdpoint1;
  VECTOR vec1;
  double bearing;
  double dist = 1.5;			    /* offset distance in degrees 
					     * (must be < 180.0) */ 
  
  /* Obtain the pointing (X-) direction on the sky from the attitude */
  rdpoint0 = rd_sky(sky_attsc(att, vector(1.0, 0.0, 0.0)));
  
  /* Offset from the X-axis by DIST degrees in the +Z direction */
  vec1 = vector(cos(RAD * dist), 0.0, sin(RAD * dist));
  rdpoint1 = rd_sky(sky_attsc(att, vec1));

  /* Use a traditional navigation formula to give the Great Circle course
   * from rdpoint0 to rdpoint1. */
  lo0 = 360.0 - rdpoint0.ra; lo1 = 360.0 - rdpoint1.ra; /* longitudes */
  la0 = rdpoint0.dec; la1 = rdpoint1.dec;               /* latitudes */
  
  if ((la0 <= -90.0) || (la0 >= +90.0))	    /* pointing at a pole */
    bearing = lo1;			    /* give the longitude at least */
  else
    bearing = DEG * acos((sin(RAD*la1) - (sin(RAD*la0))*cos(RAD*dist)) /
			 (sin(RAD*dist) * cos (RAD*la0)));

  if (sin(RAD*(lo1 - lo0)) >= 0.0)
    bearing = 360.0 - bearing;

  if (HEXTErockDebugLevel > 0)
    printf("(posz_att): +Z axis position angle = %+5.1f degrees.\n", bearing);
  return(bearing);
}
